@@ -20,8 +20,10 @@
 #include "LSDStatsTools.hpp"
 
 #include "LSD_xtensor_utils.hpp"
+#include "LSDMostLikelyPartitionsFinder.hpp"
 #include "TNT/tnt.h"
 
+
 #include <omp.h>  //Used for OpenMP run-time functions
 #ifndef _OPENMP
   #define omp_get_thread_num()  0
@@ -559,4 +561,71 @@ namespace xtlsd
 
 } // end of namespace xtlsd
 
+
+namespace mudd14partitioner
+{
+
+	std::map<std::string,xt::pytensor<float,1> > segment_data(xt::pytensor<float,1>& X_data, xt::pytensor<float,1>& Y_data, xt::pytensor<float,1>& sigmas, int min_seg_size)
+	{
+
+
+		std::vector<float> X_data_vec(X_data.size()), Y_data_vec(X_data.size()), sigmas_vec(X_data.size());
+
+		for(size_t i=0; i< X_data.size(); i++)
+		{
+			X_data_vec[i] = X_data[i];
+			Y_data_vec[i] = Y_data[i];
+			sigmas_vec[i] = sigmas[i];
+		}
+
+
+		LSDMostLikelyPartitionsFinder Partitioner(min_seg_size,X_data_vec, Y_data_vec);
+
+	    // Partition the data
+	    //float sigma = 0.0005;  // this is a placeholder. Later we can use slope uncertainties. NOW USING MEASURED ERROR
+	    // We use the standard error of the S values as the sigma in partitioner.
+	    //cout << "This basin is: " << this_basin << endl;
+	    Partitioner.best_fit_driver_AIC_for_linear_segments(sigmas_vec);
+
+	    // Now we extract all the data from the partitions
+	    vector<float> sigma_values;
+	    sigma_values.push_back(1);
+	    int node = 0;
+	    vector<float> b_values;
+	    vector<float> m_values;
+	    vector<float> r2_values;
+	    vector<float> DW_values;
+	    vector<float> fitted_y;
+	    vector<int> seg_lengths;
+	    float this_MLE;
+	    int this_n_segments;
+	    int this_n_nodes;
+	    float this_AIC;
+	    float this_AICc;
+
+	    Partitioner.get_data_from_best_fit_lines(node, sigma_values,
+	                      b_values, m_values,r2_values, DW_values, fitted_y,seg_lengths,
+	                      this_MLE,  this_n_segments,  this_n_nodes,
+	                      this_AIC,  this_AICc);
+		xt::xtensor<float,1> b_values_out = xt::adapt(b_values); 
+		xt::xtensor<float,1> m_values_out = xt::adapt(m_values); 
+		xt::xtensor<float,1> r2_values_out = xt::adapt(r2_values); 
+		xt::xtensor<float,1> DW_values_out = xt::adapt(DW_values); 
+		xt::xtensor<float,1> fitted_y_out = xt::adapt(fitted_y); 
+		xt::xtensor<float,1> seg_lengths_out = xt::adapt(seg_lengths); 
+
+		std::map<std::string,xt::pytensor<float,1> > output;
+		output["b_values"] = b_values_out;
+		output["m_values"] = m_values_out;
+		output["r2_values"] = r2_values_out;
+		output["DW_values"] = DW_values_out;
+		output["fitted_y"] = fitted_y_out;
+		output["seg_lengths"] = seg_lengths_out;
+
+		return output;
+
+	}
+
+
+}
 #endif
\ No newline at end of file
